#include <SDL3/SDL.h>
#include <SDL3/SDL_events.h>
#include <SDL3/SDL_properties.h>
#include <SDL3/SDL_render.h>
#include <SDL3/SDL_surface.h>
#include <SDL3_ttf/SDL_ttf.h>
#include <stdbool.h>
#include <stdlib.h>
#include <stdio.h>

#include "SDL_FPSCounter.h"

#define INITIAL_WINDOW_HEIGHT 600
#define INITIAL_WINDOW_WIDTH  600

void DrawNumber(SDL_Window* window, SDL_Renderer* renderer, SDL_Texture* digits[10], int number)
{
	char number_string[snprintf(NULL, 0, "%i", number) + 1];
	snprintf(number_string, sizeof(number_string), "%i", number);

	int window_height, window_width;
	SDL_GetWindowSize(window, &window_width, &window_height);

	int current_x = 0;
	char* digit_p = number_string;
	for (; *digit_p; digit_p++) {
		int index = (*digit_p) - '0';
		SDL_Texture* digit_texture = digits[index];
		SDL_FRect target_rect;
		target_rect.x = current_x;
		target_rect.y = 0;

		SDL_GetTextureSize(digit_texture, &target_rect.w, &target_rect.h);

		SDL_RenderTexture(renderer, digit_texture, NULL, &target_rect);

		current_x += target_rect.w;
	}
}

int main(int argc, char* argv[])
{
	(void) argc;
	(void) argv;

	SDL_Init( SDL_INIT_VIDEO | SDL_INIT_EVENTS);
	if (!TTF_Init()) {
		fprintf(stderr, "Initializing SDL3_ttf failed: %s\n", SDL_GetError());
	}

	SDL_Window* window = SDL_CreateWindow(
		"FPS-Counter",
		INITIAL_WINDOW_WIDTH, INITIAL_WINDOW_HEIGHT,
		0
	);
	if (window == NULL) {
		fprintf(stderr, "Creating the window failed: %s\n", SDL_GetError());
	}

	SDL_Renderer* renderer = SDL_CreateRenderer(window, NULL);
	if (renderer == NULL) {
		fprintf(stderr, "Creating the window renderer failed: %s\n", SDL_GetError());
	}


	TTF_Font* number_font = TTF_OpenFont("/usr/share/fonts/urw-base35/NimbusMonoPS-Regular.otf", 300);
	SDL_Texture* digits[10];
	for (int i = 0; i < 10; i++) {
		char string[3];
		snprintf(string, 3, "%i", i);
		SDL_Surface* digit_surface = TTF_RenderText_Solid(number_font, string, strlen(string),((SDL_Color) {0x2D, 0x2D, 0x2D, 0xFF}));
		digits[i] = SDL_CreateTextureFromSurface(renderer, digit_surface);
		SDL_DestroySurface(digit_surface);
	}
	TTF_CloseFont(number_font);


	bool continue_ = true;
	
	SDL_FPSCounter fpscounter;
	if (SDL_CreateFPSCounter(&fpscounter, 1000)) {
		fprintf(stderr, "Failed to create fps counter: %s\n", SDL_GetError());
		return 1;
	}

	while (continue_) {

		SDL_Event event;
		while(SDL_PollEvent(&event)) {
			switch (event.type) {
				case SDL_EVENT_QUIT:
					continue_ = false;		
					break;
			}
		}

		SDL_SetRenderDrawColor(renderer, 0xFF, 0xC2, 0xDE, 0xFF);
		SDL_RenderClear(renderer);

		SDL_FPSCounterTick(&fpscounter);
		DrawNumber(window, renderer, digits, SDL_FPSCounterFPS(&fpscounter));

		SDL_RenderPresent(renderer);
	}

	SDL_DestroyFPSCounter(&fpscounter);
	SDL_DestroyWindow(window);
	SDL_Quit();

	return EXIT_SUCCESS;
}
